@@ -4,7 +4,7 @@ use strict;
 use warnings;
 use Mail::Milter::Authentication::Pragmas;
 # ABSTRACT: Handler superclass
-our $VERSION = '3.20210308'; # VERSION
+our $VERSION = '3.20210324'; # VERSION
 use Mail::Milter::Authentication::Exception;
 use Mail::Milter::Authentication::Resolver;
 use Date::Format qw{ time2str };
@@ -1544,6 +1544,13 @@ sub error_dequeue($self,$id) {
 }
 
 
+sub add_header_to_sanitize_list {
+    my($self,$header,$silent) = @_;
+    return 0 if ! $self->is_handler_loaded('Sanitize');
+    return $self->get_handler('Sanitize')->add_header_to_sanitize_list($header,$silent);
+}
+
+
 sub is_local_ip_address {
     my ($self) = @_;
     return 0 if ! $self->is_handler_loaded('LocalIP');
@@ -2223,7 +2230,7 @@ Mail::Milter::Authentication::Handler - Handler superclass
 
 =head1 VERSION
 
-version 3.20210308
+version 3.20210324
 
 =head1 DESCRIPTION
 
@@ -2610,6 +2617,10 @@ Delete a previously queued item
 
 Mark a previously queued item as errored
 
+=head2 I<add_header_to_sanitize_list($header,$silent)>
+
+Add the given to the list of headers removed by the Sanitize handler if loaded
+
 =head2 I<is_local_ip_address()>
 
 Is the current connection from a local ip address?
