@@ -4,7 +4,7 @@ use strict;
 use warnings;
 use Mail::Milter::Authentication::Pragmas;
 # ABSTRACT: Example handler class
-our $VERSION = '3.20210308'; # VERSION
+our $VERSION = '3.20210324'; # VERSION
 use base 'Mail::Milter::Authentication::Handler';
 
 sub default_config {
@@ -12,9 +12,7 @@ sub default_config {
 }
 
 sub eom_callback {
-
-    # On HELO
-    my ( $self, $helo_host ) = @_;
+    my ($self) = @_;
     $self->append_header('X-Authentication-Milter','Header added by Authentication Milter');
 }
 
@@ -32,7 +30,7 @@ Mail::Milter::Authentication::Handler::AddID - Example handler class
 
 =head1 VERSION
 
-version 3.20210308
+version 3.20210324
 
 =head1 DESCRIPTION
 
