@@ -4,7 +4,7 @@ use strict;
 use warnings;
 use Mail::Milter::Authentication::Pragmas;
 # ABSTRACT: Handler class for BIMI
-our $VERSION = '3.20210308'; # VERSION
+our $VERSION = '3.20210324'; # VERSION
 use base 'Mail::Milter::Authentication::Handler';
 use Mail::BIMI 2;
 
@@ -20,57 +20,27 @@ sub default_config {
 sub register_metrics {
     return {
         'bimi_total' => 'The number of emails processed for BIMI',
-        'bimi_removed_total' => 'The number BIMI headers removed',
     };
 }
 
-sub remove_bimi_header {
-    my ( $self, $header, $value ) = @_;
-    $self->metric_count( 'bimi_removed_total' );
-    if ( !exists( $self->{'remove_bimi_headers'} ) ) {
-        $self->{'remove_bimi_headers'} = {};
-    }
-    if ( !exists( $self->{'remove_bimi_headers'}->{$header} ) ) {
-        $self->{'remove_bimi_headers'}->{$header} = [];
-    }
-    push @{ $self->{'remove_bimi_headers'}->{$header} }, $value;
+sub setup_callback {
+    my ($self) = @_;
+    my $config = $self->handler_config();
+    my $sanitize_location_header = $config->{sanitize_location_header} // 'yes';
+    my $sanitize_indicator_header = $config->{sanitize_indicator_header} // 'silent';
+    $self->add_header_to_sanitize_list('bimi-location', $sanitize_location_header eq 'silent') unless $sanitize_location_header eq 'no';
+    $self->add_header_to_sanitize_list('bimi-indicator', $sanitize_indicator_header eq 'silent') unless $sanitize_indicator_header eq 'no';
+    return;
 }
 
 sub envfrom_callback {
     my ( $self, $env_from ) = @_;
-    delete $self->{'bimi_header_index'};
-    delete $self->{'remove_bimi_headers'};
     $self->{ 'header_added' } = 0;
 }
 
 sub header_callback {
     my ( $self, $header, $value ) = @_;
 
-    # Not sure where this should go in the flow, so it's going here!
-    # Which is clearly, or at least probably the wrong place.
-    #
-    foreach my $header_type ( qw{ BIMI-Location BIMI-Indicator} ) {
-        if ( lc $header eq lc $header_type ) {
-            if ( !exists $self->{'bimi_header_index'} ) {
-                $self->{'bimi_header_index'} = {};
-            }
-            if ( !exists $self->{'bimi_header_index'}->{lc $header_type} ) {
-                $self->{'bimi_header_index'}->{lc $header_type} = 0;
-            }
-            $self->{'bimi_header_index'}->{lc $header_type} =
-            $self->{'bimi_header_index'}->{lc $header_type} + 1;
-            $self->remove_bimi_header( $header_type, $self->{'bimi_header_index'}->{lc $header_type} );
-            my $forged_header =
-              "(Received $header_type header removed by "
-              . $self->get_my_hostname()
-              . ')' . "\n"
-              . '    '
-              . $value;
-            $self->append_header( 'X-Received-'.$header_type,
-                $forged_header );
-        }
-    }
-
     return if ( $self->is_local_ip_address() );
     return if ( $self->is_trusted_ip_address() );
     return if ( $self->is_authenticated() );
@@ -102,7 +72,6 @@ sub header_callback {
         }
         $self->{'from_header'} = $value;
     }
-    ## ToDo remove/rename existing headers here
 }
 
 sub eom_requires {
@@ -120,16 +89,6 @@ sub eom_callback {
         return;
     }
 
-    # Again, not sure where this should go, so it's going here.
-    if ( exists( $self->{'remove_bimi_headers'} ) ) {
-        foreach my $header_type ( sort keys %{ $self->{'remove_bimi_headers'} } ) {
-            foreach my $header ( reverse @{ $self->{'remove_bimi_headers'}->{$header_type} } ) {
-                $self->dbgout( 'RemoveBIMIHeader', "$header_type $header", LOG_DEBUG );
-                $self->change_header( $header_type, $header, q{} );
-            }
-        }
-    }
-
     return if ( $self->{ 'header_added' } );
     return if ( $self->is_local_ip_address() );
     return if ( $self->is_trusted_ip_address() );
@@ -293,7 +252,27 @@ sub eom_callback {
                 $self->{'bimi_object'} = $BIMI; # For testing!
 
                 my $Result;
-                $Result = $BIMI->result() if ! $Skip;
+                my $timeout = $config->{'timeout'} // 5000000;
+                eval {
+                    $self->set_handler_alarm( $timeout );
+                    $Result = $BIMI->result() if ! $Skip;
+                };
+                if ( my $Error = $@ ) {
+                    $self->reset_alarm();
+                    my $Type = $self->is_exception_type( $Error );
+                    if ( $Type ) {
+                        if ( $Type eq 'Timeout' ) {
+                            # We have a timeout, is it global or is it ours?
+                            if ( $self->get_time_remaining() > 0 ) {
+                                # We have time left, but this operation save timed out
+                                $Skip = 'Timeout';
+                            }
+                            else {
+                                $self->handle_exception( $Error );
+                            }
+                        }
+                    }
+                }
 
                 if ( !$Skip
                      && $config->{rbl_no_evidence_allowlist}
@@ -355,9 +334,7 @@ sub close_callback {
     delete $self->{'selector'};
     delete $self->{'from_header'};
     delete $self->{'failmode'};
-    delete $self->{'remove_bimi_headers'};
     delete $self->{'bimi_object'};
-    delete $self->{'bimi_header_index'};
     delete $self->{'header_added'};
 }
 
@@ -375,7 +352,7 @@ Mail::Milter::Authentication::Handler::BIMI - Handler class for BIMI
 
 =head1 VERSION
 
-version 3.20210308
+version 3.20210324
 
 =head1 DESCRIPTION
 
@@ -397,6 +374,9 @@ This handler requires the DMARC handler and its dependencies to be installed and
                                                         | Allow and Block list cannot both be present
             "rbl_no_evidence_allowlist" : "",           | Optonal RBL Allow list of allowed org domains that do NOT require evidence documents
                                                         | When set, domains not on this list which do not have evidence documents will be 'skipped'
+            "timeout" : 5000000,                        | Timeout, in microseconds, to apply to a BIMI record check/fetch, detault 5000000 (5s)
+            "sanitize_location_header" : "yes",         | Remove existing BIMI-Location header? yes|no|silent (default yes)
+            "sanitize_indicator_header" : "yes",        | Remove existing BIMI-Location header? yes|no|silent (default silent)
         },
 
 =head1 AUTHOR
