@@ -4,7 +4,7 @@ use strict;
 use warnings;
 use Mail::Milter::Authentication::Pragmas;
 # ABSTRACT: Handler class for Removing headers
-our $VERSION = '3.20210308'; # VERSION
+our $VERSION = '3.20210324'; # VERSION
 use base 'Mail::Milter::Authentication::Handler';
 
 sub default_config {
@@ -64,10 +64,26 @@ sub remove_auth_header {
     push @{ $self->{'remove_auth_headers'} }, $index;
 }
 
-sub get_headers_to_remove {
-    my ( $self ) = @_;
-    my @headers = qw{ X-Disposition-Quarantine };
-    return \@headers;
+{
+    my $headers_to_remove = {
+        'x-disposition-quarantine' => { silent => 1 },
+    };
+
+    sub add_header_to_sanitize_list {
+        my ( $self, $header, $silent ) = @_;
+        $headers_to_remove->{lc $header} = { silent => $silent };
+    }
+
+    sub get_headers_to_remove {
+        my ( $self ) = @_;
+        my @headers = sort keys $headers_to_remove->%*;
+        return \@headers;
+    }
+
+    sub get_remove_header_settings {
+        my ($self, $key) = @_;
+        return $headers_to_remove->{lc $key};
+    }
 }
 
 sub envfrom_callback {
@@ -79,7 +95,7 @@ sub envfrom_callback {
     foreach my $header ( sort @{ $self->get_headers_to_remove() } ) {
         $headers->{ lc $header } = {
             'index'  => 0,
-            'silent' => 1,
+            'silent' => $self->get_remove_header_settings($header)->{silent},
         };
     }
     $self->{'header_hash'} = $headers;
@@ -99,11 +115,32 @@ sub header_callback {
         }
         $self->{'auth_result_header_index'} =
           $self->{'auth_result_header_index'} + 1;
-        my ($domain_part) = $value =~ /^([^;]*);/;
-        $domain_part =~ s/ +//g;
-        if ( $self->is_hostname_mine($domain_part) ) {
+
+        my $authserv_id = '';
+        eval {
+            my $parsed = Mail::AuthenticationResults::Parser->new()->parse($value);
+            $authserv_id = $parsed->value()->value();
+        };
+        if ( my $error = $@ ) {
+            $self->handle_exception($error);
+            $self->log_error("Error parsing existing Authentication-Results header: $error");
+        }
+
+        my $remove = 0;
+        my $silent = lc $config->{'remove_headers'} eq 'silent';
+        if ( $authserv_id ) {
+            $remove = $self->is_hostname_mine($authserv_id);
+        }
+        else {
+            # We couldn't parse the authserv_id, removing this header is the safest option
+            # Add to X-Received headers for analysis later
+            $remove = 1;
+            $silent = 0;
+        }
+
+        if ( $remove ) {
             $self->remove_auth_header( $self->{'auth_result_header_index'} );
-            if ( lc $config->{'remove_headers'} ne 'silent' ) {
+            if ( ! $silent ) {
                 my $forged_header =
                   '(Received Authentication-Results header removed by '
                   . $self->get_my_hostname()
@@ -179,7 +216,7 @@ Mail::Milter::Authentication::Handler::Sanitize - Handler class for Removing hea
 
 =head1 VERSION
 
-version 3.20210308
+version 3.20210324
 
 =head1 DESCRIPTION
 
