@@ -4,7 +4,7 @@ use strict;
 use warnings;
 use Mail::Milter::Authentication::Pragmas;
 # ABSTRACT: Class used for testing
-our $VERSION = '3.20210308'; # VERSION
+our $VERSION = '3.20210324'; # VERSION
 use Mail::Milter::Authentication;
 use Mail::Milter::Authentication::Client;
 use Mail::Milter::Authentication::Protocol::Milter;
@@ -411,6 +411,7 @@ sub send_smtp_packet {
     my ( $socket, $send, $expect ) = @_;
     print $socket "$send\r\n";
     my $recv = <$socket>;
+    $recv = '' if !defined $recv;
     while ( $recv =~ /^\d\d\d\-/ ) {
         $recv = <$socket>;
     }
@@ -418,7 +419,9 @@ sub send_smtp_packet {
         return 1;
     }
     else {
-        warn "SMTP Send expected $expect received $recv when sending $send";
+        chomp $recv;
+        chomp $send;
+        warn "SMTP Send expected \"$expect\" received \"$recv\" when sending \"$send\"\n";
         return 0;
     }
 }
@@ -573,7 +576,7 @@ Mail::Milter::Authentication::Tester - Class used for testing
 
 =head1 VERSION
 
-version 3.20210308
+version 3.20210324
 
 =head1 AUTHOR
 
